Index: src/builtins/s390/builtins-s390.cc |
diff --git a/src/builtins/s390/builtins-s390.cc b/src/builtins/s390/builtins-s390.cc |
index 4e7b585e32052142e50b48037504111e689c818e..f9aebf935048f77121d58cc380c2bf648a91458c 100644 |
--- a/src/builtins/s390/builtins-s390.cc |
+++ b/src/builtins/s390/builtins-s390.cc |
@@ -1081,12 +1081,13 @@ void Builtins::Generate_InterpreterEntryTrampoline(MacroAssembler* masm) { |
FieldMemOperand(debug_info, DebugInfo::kDebugBytecodeArrayIndex)); |
__ bind(&array_done); |
- // Check function data field is actually a BytecodeArray object. |
- Label bytecode_array_not_present; |
- __ CompareRoot(kInterpreterBytecodeArrayRegister, |
- Heap::kUndefinedValueRootIndex); |
- __ beq(&bytecode_array_not_present); |
+ // Check whether we should continue to use the interpreter. |
+ Label switch_to_different_code_kind; |
+ __ LoadP(r2, FieldMemOperand(r2, SharedFunctionInfo::kCodeOffset)); |
+ __ CmpP(r2, Operand(masm->CodeObject())); // Self-reference to this code. |
+ __ bne(&switch_to_different_code_kind); |
+ // Check function data field is actually a BytecodeArray object. |
if (FLAG_debug_code) { |
__ TestIfSmi(kInterpreterBytecodeArrayRegister); |
__ Assert(ne, kFunctionDataShouldBeBytecodeArrayOnInterpreterEntry); |
@@ -1152,10 +1153,10 @@ void Builtins::Generate_InterpreterEntryTrampoline(MacroAssembler* masm) { |
LeaveInterpreterFrame(masm, r4); |
__ Ret(); |
- // If the bytecode array is no longer present, then the underlying function |
- // has been switched to a different kind of code and we heal the closure by |
- // switching the code entry field over to the new code object as well. |
- __ bind(&bytecode_array_not_present); |
+ // If the shared code is no longer this entry trampoline, then the underlying |
+ // function has been switched to a different kind of code and we heal the |
+ // closure by switching the code entry field over to the new code as well. |
+ __ bind(&switch_to_different_code_kind); |
__ LeaveFrame(StackFrame::JAVA_SCRIPT); |
__ LoadP(r6, FieldMemOperand(r3, JSFunction::kSharedFunctionInfoOffset)); |
__ LoadP(r6, FieldMemOperand(r6, SharedFunctionInfo::kCodeOffset)); |